/*
This software is available under NYSL(Nirunari Yakunari Sukinishiro License).
*/

#include <SPI.h>

/*GR-(RL78) Sketch Template Version: E0.80*/
#include <RLduino78.h>
#include <math.h>
#include "types.h"
#include "common.h"


// Pin 22,23,24 are assigned to RGB LEDs.
/*
int led_red   = 22; // LOW active
int led_green = 23; // LOW active
int led_blue  = 24; // LOW active
*/

// 0x02/0x03だとデバッグしにくいので、#/$ を 電文開始/終了 に使う
#define CODE_STX '#'    // 電文開始
#define CODE_ETX '$'    // 電文終了

// ポート
#define SW_RIGHT    4  // 右ジョイスティックスイッチ入力
#define SW_LEFT     5  // 左ジョイスティックスイッチ入力
#define SS_RIGHT    6  // 右加速度センサSPIスレーブセレクト出力
#define SS_LEFT     9  // 左加速度センサSPIスレーブセレクト出力
#define AD_RIGHT_X  1  // 右ジョイスティックX軸アナログ入力
#define AD_RIGHT_Y  0  // 右ジョイスティックY軸アナログ入力
#define AD_LEFT_X   3  // 左ジョイスティックX軸アナログ入力
#define AD_LEFT_Y   2  // 左ジョイスティックY軸アナログ入力

// ADXL362のレジスタに書き込む
void ADXL362_write(int ss, byte addr, byte val)
{
    digitalWrite(ss, LOW);
    SPI.transfer(0x0A);    // write
    SPI.transfer(addr);
    SPI.transfer(val);
    digitalWrite(ss, HIGH);
}

// ADXL362のレジスタを読み出す
byte ADXL362_read(int ss, byte addr){
    byte val = 0;
    
    digitalWrite(ss, LOW);
    SPI.transfer(0x0B);    // read
    SPI.transfer(addr);
    val = SPI.transfer(0x00);
    digitalWrite(ss, HIGH);

    return val;
}

// ADXL362を初期化する
void ADXL362_init(int ss)
{
    // ソフトリセット
    ADXL362_write(ss, 0x1F, 0x52);
    delay(10);
    
    // パワーコントロールレジスタ：測定モード
    ADXL362_write(ss, 0x2D, 0x02);
    delay(10);
}

// ADXL362から加速度センサの読み取り
vector3D ADXL362_readAcc(int ss)
{
    uint16_t x,y,z;
    vector3D acc;
    
    // burst SPI read
    digitalWrite(ss, LOW);
    SPI.transfer(0x0B); // アドレス0Ehから連続読み出し
    SPI.transfer(0x0E); // XDATA_L,XDATA_H,YDATA_L,YDATA_H,ZDATA_L,ZDATA_H
    x  =  (uint16_t)SPI.transfer(0x00);
    x |= ((uint16_t)SPI.transfer(0x00) << 8);
    y  =  (uint16_t)SPI.transfer(0x00);
    y |= ((uint16_t)SPI.transfer(0x00) << 8);
    y = 1-y;  // Y軸は極性反転
    z  =  (uint16_t)SPI.transfer(0x00);
    z |= ((uint16_t)SPI.transfer(0x00) << 8);
    z = 1-z;  // Z軸は極性反転
    digitalWrite(ss, HIGH);
    
    // デバッグ用 16ビット値
    acc.ux = x;
    acc.uy = y;
    acc.uz = z;
    
    // ±ありの浮動小数点実数に換算
    acc.x = (double)((int16_t)x);
    acc.y = (double)((int16_t)y);
    acc.z = (double)((int16_t)z);
    
    return acc;
}

// ジョイスティックあそび処理, ±90 -> 0-180, クリップ
unsigned short joystick_deadzone(long x, long dead)
{
    long fx;
    unsigned short ret;
    
    if(abs(x) < dead){
        fx = 0;
    }else if( x > 0 ){
        fx = x - dead;
    }else{
        fx = x + dead;
    }
    
    fx += 90;
    if(fx < 0)   fx = 0;
    if(fx > 180) fx = 180;
    
    ret = (unsigned short)fx;
    return ret;
}

// 初期設定
void setup()
{
    // シリアルの設定
    Serial1.begin(19200);
    
    // SPIの設定
    pinMode(10,       OUTPUT);
    pinMode(SS_RIGHT, OUTPUT);
    pinMode(SS_LEFT,  OUTPUT);
    digitalWrite(SS_RIGHT, HIGH);
    digitalWrite(SS_LEFT,  HIGH);
    SPI.begin();
    SPI.setClockDivider(SPI_CLOCK_DIV128);
    SPI.setBitOrder(MSBFIRST);
    SPI.setDataMode(SPI_MODE0);

    // GPIOの設定
    pinMode(SW_RIGHT, INPUT_PULLUP); // 要プルアップ
    pinMode(SW_LEFT,  INPUT_PULLUP); // 要プルアップ
    
    // 加速度センサの初期化
    ADXL362_init(SS_RIGHT);
    ADXL362_init(SS_LEFT);
    
    // ウェイト
    delay(2000);
/*
    pinMode(led_red, OUTPUT);
    pinMode(led_green, OUTPUT);
    pinMode(led_blue, OUTPUT);
    digitalWrite(led_red, HIGH);
    digitalWrite(led_green, HIGH);
    digitalWrite(led_blue, HIGH);
*/
}

// メインループ
void loop()
{
    static char hoge[64];  
    /*
     * センサの読み出し
     */
    
    // アナログポート(ジョイスティック)を読む
    long adrx = analogRead(AD_RIGHT_X); 
    long adry = analogRead(AD_RIGHT_Y);
    long adlx = analogRead(AD_LEFT_X);
    long adly = analogRead(AD_LEFT_Y);
    
    // スイッチ(ジョイスティック)を読む
    int swr = digitalRead(SW_RIGHT);
    int swl = digitalRead(SW_LEFT);
    // 加速度センサを読む
    vector3D accr = ADXL362_readAcc( SS_RIGHT );
    vector3D accl = ADXL362_readAcc( SS_LEFT );
    
    /*
     * データの換算
     */
    // ジョイスティック→腕サーボ角度 (0-1023 -> ±90)
    adrx = ((adrx-512L) * 200L) / 1023L;
    adry = ((adry-512L) * 200L) / 1023L;
    adlx = ((adlx-512L) * 200L) / 1023L;
    adly = ((adly-512L) * 200L) / 1023L;    
    
    // ジョイスティックあそび処理, ±90 -> 0-180, クリップ
    unsigned short joy_rx = joystick_deadzone(adrx, 10);
    unsigned short joy_ry = joystick_deadzone(adry, 10);
    unsigned short joy_lx = joystick_deadzone(adlx, 10);
    unsigned short joy_ly = joystick_deadzone(adly, 10);
    
    // 右手加速度→移動方向と速度
    char dir;
    unsigned short vel;
    if( swr == 0 )
    {
        // θy = atan(Ax/Az), θy = atan(Ay/Az)
        double rl = atan2( accr.x, accr.z ) * 10.0 / (M_PI/2.0);
        double fb = atan2( accr.y, accr.z ) * 10.0 / (M_PI/2.0);
//        sprintf(hoge,"%5d %5d\n",(int)fb,(int)rl);
//        Serial1.write(hoge);
        if( abs(rl) > abs(fb) ){
            if(rl>1){
                dir = 'R';
                vel = (unsigned short)rl+3;
            }else if(rl<-1){
                dir = 'L';
                vel = (unsigned short)(-rl)+3;
            }else{
                dir = 'S';
                vel = 0;
            }
        }else{
            if(fb > 1){
                dir = 'F';
                vel = (unsigned short)fb+3;
            }else if(fb<-1){
                dir = 'B';
                vel = (unsigned short)(-fb)+3;
            }else{
                dir = 'S';
                vel = 0;
            }
        }
        if(vel>9) vel = 9;
    }else{
        // 右手のスイッチが押されていないときは無効
        dir = 'S';
        vel = 0;
    }
    // 左手加速度→首・腰のサーボ角度
    unsigned short neck, waist;
    if( swl == 0 )
    {
        // θy = atan(Ax/Az), θy = atan(Ay/Az)
        long fNeck  = long( atan2( accl.x, accl.z ) * 1.2 * 180.0 / M_PI );
        long fWaist = long( atan2( accl.y, accl.z ) * 1.2 * 180.0 / M_PI );
        
        if(abs(fNeck) > abs(fWaist))
        {
            waist = 90;
            neck  = joystick_deadzone(fNeck, 12);
        }else{
            waist = joystick_deadzone(fWaist, 12);
            neck  = 90;
        }
    }else{
        // 左手のスイッチが押されていないときは無効
        neck  = 90;
        waist = 90;
    }
    
#if 1
    /*
     * コマンド文字列の生成
     */
    // 01234567890123456789012345
    // #AAABBBCCCDDDEEEFFFGH$
    // AAA=Waist BBB=Neck CCC=Arm-L1 DDD=Arm-L2
    // EEE=Arm-R1 FFF=Arm-R2 G=方向[FBRLS] H=速度(0-9)
    static char command[32];
    command[0]  = CODE_STX;
    Uint16ToDec(&command[1],  waist,  3);
    Uint16ToDec(&command[4],  neck,   3);
    Uint16ToDec(&command[7],  joy_ly, 3);
    Uint16ToDec(&command[10], joy_lx, 3);
    Uint16ToDec(&command[13], joy_ry, 3);
    Uint16ToDec(&command[16], joy_rx, 3);
    command[19] = dir;
    Uint16ToDec(&command[20], vel, 1);
    command[21] = CODE_ETX;
    command[22] = '\n'; // コンソールでの見やすさのため改行を追加(処理に影響しない)
    // コマンド送信
    Serial1.write((uint8_t*)command, 23);
#else
    /* テスト用 */
    static char command[64];
    command[0] = 'L';
    command[1] = (swl) ? '1' : '0';
    command[2] = 'R';
    command[3] = (swr) ? '1' : '0';
    command[4] = 'X';
    Uint16ToDec(&command[5], (unsigned short)adlx, 4);
    command[9] = 'Y';
    Uint16ToDec(&command[10], (unsigned short)adly, 4);
    command[14] = 'X';
    Uint16ToDec(&command[15], (unsigned short)adrx, 4);
    command[19] = 'Y';
    Uint16ToDec(&command[20], (unsigned short)adry, 4);
    command[24] = '_';
    command[25] = 'X';
    Uint16ToHex(&command[26], accl.ux, 4);
    command[30] = 'Y';
    Uint16ToHex(&command[31], accl.uy, 4);
    command[35] = 'Z';
    Uint16ToHex(&command[36], accl.uz, 4);

    command[40] = '_';
    command[41] = 'X';
    Uint16ToHex(&command[42], accr.ux, 4);
    command[46] = 'Y';
    Uint16ToHex(&command[47], accr.uy, 4);
    command[51] = 'Z';
    Uint16ToHex(&command[52], accr.uz, 4);
    command[56] = '\n';
    // コマンド送信
    Serial1.write((uint8_t*)command, 57);
#endif
    
    // ウェイト
    delay(100);
//    delay(500);
}
